@@ -1,5 +1,4 @@
 """ **Description**
-
         A rank filter realizes a nonlinear morphological operator consuming an
         incident signal, sorting, indexing, and selecting over a sliding window
         of a specified order, and producing a reference signal.
@@ -12,134 +11,90 @@
         sequential erosion and dilation.  An order and rank are specified.
 
         .. math::
-
             y_{n} = sort(\ x_{n-N+1\ :\ n}\ )[\ i\ ]
 
     **Example**
-
+       
         ::
-
+        
             from diamondback import RankFilter
             import numpy
 
-
-            # Create an instance with rank and order.
+            # Create an instance.
 
             obj = RankFilter( rank = 4, order = 4 )
 
             # Filter an incident signal.
 
             x = numpy.concatenate( ( numpy.ones( 1 ), numpy.zeros( 10 ), numpy.ones( 4 ), numpy.zeros( 2 ), numpy.ones( 5 ), numpy.zeros( 6 ) ) )
-
             obj.reset( x[ 0 ] )
-
             y = obj.filter( x )
 
     **License**
-
         `BSD-3C.  <https://github.com/larryturner/diamondback/blob/master/license>`_
-
-        Copyright (c) 2018, Larry Turner, Schneider Electric.  All rights reserved.
+        © 2018 - 2023 Larry Turner, Schneider Electric Industries SAS. All rights reserved.
 
     **Author**
-
-        Larry Turner, Schneider Electric, Analytics & AI, 2018-01-31.
-
-    **Definition**
-
+        Larry Turner, Schneider Electric, AI Hub, 2018-01-31.
 """
 
 from diamondback.filters.FirFilter import FirFilter
+from typing import List, Union
 import numpy
 
-
 class RankFilter( FirFilter ) :
 
     """ Rank filter.
     """
 
     @property
-    def rank( self ) :
+    def index( self ) :
 
-        """ rank : int - in [ 0, order ].
+        """ index : int - in [ 0, order ].
         """
 
-        return self._rank
+        return self._index
 
-    @rank.setter
-    def rank( self, rank : int ) :
+    @index.setter
+    def index( self, index : int ) :
 
-        if ( ( rank < 0 ) or ( rank > ( len( self.s ) - 1 ) ) ) :
+        if ( ( index < 0 ) or ( index > ( len( self.s ) - 1 ) ) ) :
+            raise ValueError( f'Index = {index} Expected Index in [ 0, {len( self.s ) - 1} ]' )
+        self._index = index
 
-            raise ValueError( f'Rank = {rank}' )
-
-        self._rank = rank
-
-    def __eq__( self, other : any ) -> bool :
-
-        """ Equal.
-
-            Arguments :
-
-                other : any.
-
-            Returns :
-
-                equal : bool.
-        """
-
-        return ( ( super( ).__eq__( other ) ) and ( self.rank == other.rank ) )
-
-    def __init__( self, rank : int, order : int ) -> None :
+    def __init__( self, index : int, order : int ) -> None :
 
         """ Initialize.
 
             Arguments :
-
-                rank : int - in [ 0, order ].
-
+                index : int - in [ 0, order ].
                 order : int.
         """
 
-        if ( ( rank < 0 ) or ( rank > order ) ) :
+        if ( ( index < 0 ) or ( index > order ) ) :
+            raise ValueError( f'Index = {index} Order = {order} Expected Index in [ 0, {order} ]' )
+        super( ).__init__( b = numpy.ones( order + 1 ) / ( order + 1 ) )
+        self._index = index
 
-            raise ValueError( f'Rank = {rank} Order = {order}' )
-
-        super( ).__init__( numpy.ones( order + 1 ) / ( order + 1 ) )
-
-        self._rank = rank
-
-    def filter( self, x : any, d : any = None ) -> any :
+    def filter( self, x : Union[ List, numpy.ndarray ] ) -> numpy.ndarray :
 
         """ Filters an incident signal and produces a reference signal.
 
             Arguments :
-
-                x : typing.Union[ numpy.ndarray, list ] - incident signal.
+                x : Union[ List, numpy.ndarray ] - incident signal.
 
             Returns :
-
                 y : numpy.ndarray - reference signal.
         """
 
         if ( ( not numpy.isscalar( x ) ) and ( not isinstance( x, numpy.ndarray ) ) ) :
-
             x = numpy.array( list( x ) )
-
-        if ( ( len( x.shape ) != 1 ) or ( len( x ) == 0 ) ) :
-
+        if ( not len( x ) ) :
             raise ValueError( f'X = {x}' )
-
         y = numpy.zeros( len( x ), type( x[ 0 ] ) )
-
         for ii in range( 0, len( x ) ) :
-
             self.s[ 0 ] = x[ ii ]
-
-            y[ ii ] = self.s[ numpy.argsort( abs( self.s ) ) ][ self.rank ]
-
+            y[ ii ] = self.s[ numpy.argsort( abs( self.s ) ) ][ self.index ]
             if ( len( self.s ) > 1 ) :
-
                 self.s[ 1 : ] = self.s[ : -1 ]
-
         return y
